// SPDX-FileCopyrightText: Deutsches Elektronen-Synchrotron DESY, MSK, ChimeraTK Project <chimeratk-support@desy.de>
// SPDX-License-Identifier: LGPL-3.0-or-later
#pragma once

#include "Module.h"

#include <ChimeraTK/RegisterPath.h>

#include <boost/thread.hpp>

#include <list>

namespace ChimeraTK {

  /** A virtual module generated by EntityOwner::findTag(). */
  class VirtualModule : public Module {
   public:
    /** Constructor */
    VirtualModule(const std::string& name, const std::string& description, ModuleType moduleType);

    /** Copy constructor */
    VirtualModule(const VirtualModule& other);

    /** Assignment operator */
    VirtualModule& operator=(const VirtualModule& other);

    /** Destructor */
    virtual ~VirtualModule();

    VariableNetworkNode operator()(const std::string& variableName) const override;

    Module& operator[](const std::string& moduleName) const override;

    void connectTo(const Module& target, VariableNetworkNode trigger = {}) const override;

    /** Add an accessor. The accessor instance will be added to an internal list.
     */
    void addAccessor(VariableNetworkNode accessor);

    /** Add a virtual sub-module. The module instance will be added to an internal
     * list. */
    void addSubModule(VirtualModule module);

    /** Remove a virtual sub-module with the given name if it currently exists. */
    void removeSubModule(const std::string& name);

    /** Return the submodule with the given name. If it doesn't exist, create it
     * first. */
    VirtualModule& createAndGetSubmodule(const RegisterPath& moduleName);

    /** Like createAndGetSubmodule(), but recursively create a hierarchy of
     * submodules separated by "/" in the moduleName. */
    VirtualModule& createAndGetSubmoduleRecursive(const RegisterPath& moduleName);

    ModuleType getModuleType() const override { return _moduleType; }

    const Module& virtualise() const override;

    void stripEmptyChildsRecursive();

   protected:
    std::list<VirtualModule> submodules;
    ModuleType _moduleType;
  };

} /* namespace ChimeraTK */
